package ca.odell.glazedlists.example;

import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.GlazedLists;
import ca.odell.glazedlists.gui.TableFormat;
import ca.odell.glazedlists.swing.EventTableModel;

import javax.swing.*;
import java.awt.*;
import java.util.Random;
import java.util.ListIterator;
import java.util.Iterator;

public class ConcurrencyExample {

    public static class AmericanIdol {
        private String name;
        private int votes;

        public AmericanIdol(String name, int votes) {
            this.name = name;
            this.votes = votes;
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public int getVotes() {
            return votes;
        }

        public void setVotes(int votes) {
            this.votes = votes;
        }

        public void incrementVotes() {
            this.votes++;
        }
    }

    public static void main(String[] args) throws InterruptedException {
        // create an EventList of AmericanIdol
        EventList idols = new BasicEventList();
        idols.add(new AmericanIdol("Simon Cowell", 0));
        idols.add(new AmericanIdol("Paula Abdul", 0));
        idols.add(new AmericanIdol("Randy Jackson", 0));
        idols.add(new AmericanIdol("Ryan Seacrest", 0));

        idols = GlazedLists.threadSafeList(idols);

        // build a JTable
        String[] propertyNames = new String[] {"name", "votes"};
        String[] columnLabels = new String[] {"Name", "Votes"};
        TableFormat tf = GlazedLists.tableFormat(AmericanIdol.class, propertyNames, columnLabels);
        JTable t = new JTable(new EventTableModel(idols, tf));

        // place the table in a JFrame
        JFrame f = new JFrame();
        f.setLayout(new BorderLayout());
        f.add(new JScrollPane(t), BorderLayout.CENTER);

        // show the frame
        f.pack();
        f.setLocationRelativeTo(null);
        f.setVisible(true);

        // vote one idol off each episode
        while (idols.size() > 1) {
            // clear the votes
            for (ListIterator i = idols.listIterator(); i.hasNext();) {
                AmericanIdol idol = (AmericanIdol) i.next();
                idol.setVotes(0);
                i.set(idol);
            }

            // simulate voting
            long quittingTime = System.currentTimeMillis() + 5000;
            Random dice = new Random();
            while (System.currentTimeMillis() < quittingTime) {
                int index = dice.nextInt(idols.size());
                AmericanIdol idol = (AmericanIdol) idols.get(index);
                idol.incrementVotes();
                idols.set(index, idol);
            }

            // find the idol with the least votes
            AmericanIdol votedOff = (AmericanIdol) idols.get(0);
            for (Iterator i = idols.iterator(); i.hasNext();) {
                AmericanIdol idol = (AmericanIdol) i.next();
                if (idol.getVotes() < votedOff.getVotes()) {
                    votedOff = idol;
                }
            }

            // vote off the idol with the least votes
            Thread.sleep(1000);
            idols.remove(votedOff);
            Thread.sleep(1000);
        }
    }
}